<!doctype html>
<html lang="en" dir="ltr">
<head>
  <link href="https://tabler.github.io/tabler/assets/css/dashboard.css" rel="stylesheet">
</head>
<body>
  <div class="page">
    <div class="page-single">
      <div class="container">
        <div class="row">
          <div class="col col-login mx-auto">
              <div class="card-body p-2">
                <div class="alert text-center btn-info">
                  <i class="fe fe-mail" ></i> <i class="fe fe-check" ></i>
                      <h1>Un email vous a été envoyé pour vous connecter.</h1> Si vous ne le recevez pas, vous pouvez réessayer.
                </div>
                <div class="text-center">
                  <a href="{% url 'magicauth-login' %}?next={{ next_view }}" class="btn btn-secondary">Réessayer</a>
                </div>
              </div>
          </div>
        </div>
      </div>
    </div>
  </div>
</body>
</html>
